using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.Xna.Framework;
using Singleton.CCellManager;
using Singleton.CVirusManager;
using Singleton.CProteinManager;
using Singleton.CSound;

namespace CWorld
{
    public class CInfectedCell : ICell
    {
        public void determineAction( CCell thisCell, GameTime time)
        {
            // stops the entity moving
            thisCell.EntityDirection = Vector2.Zero;

            if ( thisCell.BegunInfectedTime + (thisCell.pDEFAULT_INFECTED_TIME / thisCell.NumberOfViruses) < time.TotalRealTime.TotalMilliseconds)
            {
                // 'kills' the entity
                thisCell.IsActive = false;
                
                // spawn two new viruses
                float spawnX = ((float) CCellManager.getInstance.RandomNum.NextDouble() * 2) - 1;
                float spawnY = ((float) CCellManager.getInstance.RandomNum.NextDouble() * 2) - 1;

                CVirusManager.getInstance.findFreeVirus(thisCell.CurrentPosition, new Vector2(spawnX, spawnY), time);
                CVirusManager.getInstance.findFreeVirus(thisCell.CurrentPosition, new Vector2(-spawnX, -spawnY), time);
            }
        }

        public void collisionResult(CCell thisCell, int otherEntityID)
        {
            // dies if hit by a player
            if (otherEntityID == 0)
            {
                thisCell.IsActive = false;
                CProteinManager.getInstance.spawnProtein(thisCell.CurrentPosition, thisCell.PointValue, thisCell.ExperienceGiven, 20);
            }
            else if (CWorld.Game1.entityList[otherEntityID].EntitySide == CWorldEntity.eSide.projectile)
            {
                // reduces the infection if hit by a virus
                thisCell.CellBehaviour = new CInfectingCell();
                thisCell.InfectionLevel -= ((float)CWorld.Game1.entityList[otherEntityID].CollisionValue / 100.0f);
            }
            else if (CVirusManager.getInstance.isVirus(otherEntityID))
            {
                // increases the infection rate
                thisCell.NumberOfViruses++;
            }
        }
    }
}