using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Microsoft.Xna.Framework;
using Singleton.CProteinManager;
using Singleton.CVirusManager;
using Singleton.CSound;

namespace CWorld
{
    public class CInfectingCell : ICell
    {
        public void determineAction(CCell thisCell, GameTime time)
        {
            // stops the entity moving
            thisCell.EntityDirection = Vector2.Zero;

            if (thisCell.InfectionLevel >= 1.0f)
            {
                // if cell is fully infected, then change to an infected cell
                thisCell.CellBehaviour = new CInfectedCell();
                thisCell.InfectionLevel = 1.0f;
                thisCell.BegunInfectedTime = (float) time.TotalRealTime.TotalMilliseconds;
                
            }
            else if (thisCell.InfectionLevel < 0.0f)
            {
                // spawns a protein
                CProteinManager.getInstance.spawnProtein(thisCell.CurrentPosition, thisCell.PointValue + (thisCell.NumberOfViruses * thisCell.PointValue), thisCell.ExperienceGiven + (thisCell.NumberOfViruses * thisCell.ExperienceGiven), 20);

                // if the cell is fully cured, it restores to a roaming cell
                thisCell.CellBehaviour = new CRoamingCell();
                thisCell.InfectionLevel = 0.0f;
                thisCell.NumberOfViruses = 0;
            }
            else
            {
                // progresses the infection of the cell
                thisCell.InfectionLevel += (thisCell.InfectionRate * thisCell.NumberOfViruses);
            }
        }

        public void collisionResult(CCell thisCell, int otherEntityID)
        { 
            if (otherEntityID == 0)
            {
                thisCell.IsActive = false;
                CProteinManager.getInstance.spawnProtein(thisCell.CurrentPosition, thisCell.PointValue, thisCell.ExperienceGiven, 20);
            }
            else if (CWorld.Game1.entityList[otherEntityID].EntitySide == CWorldEntity.eSide.projectile)
            {
                thisCell.InfectionLevel -= ((float) CWorld.Game1.entityList[otherEntityID].CollisionValue / 100.0f);
            }
            else if (CWorld.Game1.entityList[otherEntityID].EntitySide == CWorldEntity.eSide.virus)
            {
                // increases the infection rate
                thisCell.NumberOfViruses++;
            }
        }
    }
}