﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using Microsoft.Xna.Framework;

using Singleton.CVirusManager;
using Singleton.CCellManager;

namespace CWorld
{
    class CRoamingCell : ICell
    {
        #region fields and their properties
        private float lastDirectionChange;

        public float LastDirectionChange
        {
            get { return lastDirectionChange; }
            set { lastDirectionChange = value; }
        }

        private float DIRECTION_CHANGE_DELAY = 1000;

        public float pDIRECTION_CHANGE_DELAY
        {
            get { return DIRECTION_CHANGE_DELAY; }
            set { DIRECTION_CHANGE_DELAY = value; }
        }

        private float driftingX = 0;

        public float DriftingX
        {
            get { return driftingX; }
            set { driftingX = value; }
        }

        private float driftingY = 0;

        public float DriftingY
        {
            get { return driftingY; }
            set { driftingY = value; }
        }
        #endregion

        // determines the action of the cell based on it's current state
        public void determineAction(CCell thisCell, GameTime time)
        {
            // holds the distance to the current target to reduce the
            // amount of times the function is called
            int distanceToTarget = thisCell.EntityVision;

            #region finding the closest enemy to this entity
            // loops through every entity
            for (int i = 0; i < CWorld.Game1.entityList.Count; i++)
            {
                // prevents an entity checking against itself
                if (thisCell != CWorld.Game1.entityList[i])
                {
                    // checks to see if the other entity is active
                    if (CWorld.Game1.entityList[i].IsActive)
                    {
                        // checks to see if the other entity is an enemy
                        if (thisCell.isEnemyOf(CWorld.Game1.entityList[i].EntitySide))
                        {
                            // checks to see if the entity is range
                            if (thisCell.inRange(CWorld.Game1.entityList[i].CurrentPosition, thisCell.EntityVision))
                            {
                                // checks to see if the entity is attackable
                                if (CWorld.Game1.entityList[i].Attackable)
                                {
                                    // checks to see if this entity is closer than the current target
                                    if (thisCell.distanceBetween(CWorld.Game1.entityList[i].CurrentPosition) < distanceToTarget)
                                    {
                                        thisCell.CurrentTargetID = i;
                                        distanceToTarget = thisCell.distanceBetween(CWorld.Game1.entityList[i].CurrentPosition);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            #endregion

            if (thisCell.CurrentTargetID >= 0)
            {
                thisCell.CellBehaviour = new CAttackingCell();
            }
            else
            {
                // drifting behaviour
                if (lastDirectionChange + DIRECTION_CHANGE_DELAY < (float)time.TotalRealTime.TotalMilliseconds)
                {
                    // sets X
                    if (thisCell.CurrentPosition.X < 75)
                    {
                        driftingX = (float)(CCellManager.getInstance.RandomNum.NextDouble());
                    }
                    else if (thisCell.CurrentPosition.X > 675)
                    {
                        driftingX = (float)(CCellManager.getInstance.RandomNum.NextDouble() - 1);
                    }
                    else
                    {
                        driftingX = (float)((CCellManager.getInstance.RandomNum.NextDouble() * 2) - 1);
                    }

                    // sets Y
                    if (thisCell.CurrentPosition.Y < 75)
                    {
                        driftingY = (float)(CCellManager.getInstance.RandomNum.NextDouble());
                    }
                    else if (thisCell.CurrentPosition.Y > 525)
                    {
                        driftingY = (float)(CCellManager.getInstance.RandomNum.NextDouble() - 1);
                    }
                    else
                    {
                        driftingY = (float)((CCellManager.getInstance.RandomNum.NextDouble() * 2) - 1);
                    }

                    DIRECTION_CHANGE_DELAY = CCellManager.getInstance.RandomNum.Next(500, 1500);
                    lastDirectionChange = (float)time.TotalRealTime.TotalMilliseconds;
                }

                thisCell.EntityDirection += new Vector2(driftingX, driftingY);

                thisCell.EntityDirection += thisCell.avoid();
            }
        }

        public void collisionResult(CCell thisCell, int otherEntityID)
        {
            // if a virus collides with this entity, it becomes an infecting cell
            if (CVirusManager.getInstance.isVirus(otherEntityID))
            {
                thisCell.CellBehaviour = new CInfectingCell();
                thisCell.Attackable = false;
            }
            else if(CWorld.Game1.entityList[otherEntityID].EntitySide != CWorldEntity.eSide.projectile)
            {
                thisCell.takeDamage(CWorld.Game1.entityList[otherEntityID].CollisionValue);
            }
        }
    }
}