﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Microsoft.Xna.Framework;
using CWorld;

namespace Singleton.CCellManager
{
    class CCellManager
    {
        #region instance code
        private static CCellManager instanceCMANAGER;

        public static void createInstance(int newCellStart, int newCellCount)
        {
            if (instanceCMANAGER == null)
            {
                instanceCMANAGER = new CCellManager(newCellStart, newCellCount);
            }
        }

        public static CCellManager getInstance
        {
            get
            {
                if (instanceCMANAGER != null)
                {
                    return instanceCMANAGER;
                }
                else
                {
                    throw new Exception("Singleton not initialised.");
                }
            }
        }
        #endregion

        #region fields and their properties
        private int firstCell;

        public int FirstCell
        {
            get { return firstCell; }
            set { firstCell = value; }
        }

        private int cellCount;

        public int CellCount
        {
            get { return cellCount; }
            set { cellCount = value; }
        }

        // random number generator
        Random randomNum = new Random();

        public Random RandomNum
        {
            get { return randomNum; }
            set { randomNum = value; }
        }
        #endregion

        #region constructor
        public CCellManager(int newFirstCell, int newCellCount)
        {
            firstCell = newFirstCell;
            CellCount = newCellCount;
        }
        #endregion

        #region methods
        private bool checkInfected(CCell thisCell)
        {
            return (thisCell.InfectionLevel > 0.0f && thisCell.IsActive);
        }

        private bool checkUninfected(CCell thisCell)
        {
            return (thisCell.InfectionLevel == 0.0f && thisCell.IsActive);
        }

        public void findFreeCell(Vector2 position)
        {
            Trace.Assert(CWorld.Game1.entityList.Count != 0, "Empty list passed.");

            for (int i = firstCell; i < firstCell + cellCount; i++)
            {
                // checks for an inactive cell and breaks the loop
                // if one is found
                if (!CWorld.Game1.entityList[i].IsActive)
                {
                    spawnCell((CCell) CWorld.Game1.entityList[i], position);
                    break;
                }
            }
        }

        public void findFreeCell(Vector2 position, Vector2 direction)
        {
            Trace.Assert(CWorld.Game1.entityList.Count != 0, "Empty list passed.");

            for (int i = firstCell; i < firstCell + cellCount; i++)
            {
                // checks for an inactive cell and breaks the loop
                // if one is found
                if (!CWorld.Game1.entityList[i].IsActive)
                {
                    spawnCell((CCell)CWorld.Game1.entityList[i], position, direction);
                    break;
                }
            }
        }

        // returns whether the ID provided is one belonging to a cell
        public bool isCell(int ID)
        {
            return ((ID >= firstCell) && (ID <= firstCell + cellCount));
        }

        // returns the amount of infected cells remaining
        public int infectedCellsRemaining()
        {
            int count = 0;

            for (int i = firstCell; i < firstCell + cellCount; i++)
            {
                if (checkInfected((CCell) CWorld.Game1.entityList[i]))
                {
                    count++;
                }
            }

            return count;
        }

        private void spawnCell(CCell cell, Vector2 position)
        {
            try
            {
                // sets the cell to spawn from the set location
                cell.CurrentPosition = position;
                cell.PreviousPosition = position;
                cell.Health = cell.OrignalHealth;
                cell.updateCollisionRectangle(cell.CurrentPosition);
                cell.CollisionValue = cell.pDEFAULT_COLLISION_VALUE;
                cell.IsActive = true;
                cell.Attackable = true;
                cell.CurrentTargetID = -1;
                cell.CellBehaviour = new CRoamingCell();
                cell.InfectionLevel = 0.0f;
                cell.NumberOfViruses = 0;
            }
            catch
            {
                throw new Exception("Failure to spawn new cell.");
            }
        }

        private void spawnCell(CCell cell, Vector2 position, Vector2 direction)
        {
            try
            {
                // sets the cell to spawn from the set location
                cell.CurrentPosition = position;
                cell.PreviousPosition = position;
                cell.EntityDirection = direction;
                cell.Health = cell.OrignalHealth;
                cell.updateCollisionRectangle(cell.CurrentPosition);
                cell.CollisionValue = cell.pDEFAULT_COLLISION_VALUE;
                cell.IsActive = true;
                cell.Attackable = true;
                cell.CurrentTargetID = -1;
                cell.CellBehaviour = new CRoamingCell();
                cell.InfectionLevel = 0.0f;
                cell.NumberOfViruses = 0;
            }
            catch
            {
                throw new Exception("Failure to spawn new cell.");
            }
        }

        // returns the amount of uninfected cells remaining
        public int uninfectedCellsRemaining()
        {
            int count = 0;

            for (int i = firstCell; i < firstCell + cellCount; i++)
            {
                if (checkUninfected((CCell)CWorld.Game1.entityList[i]))
                {
                    count++;
                }
            }

            return count;
        }
        #endregion
    }
}
