﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Microsoft.Xna.Framework;
using CWorld;

namespace Singleton.CVirusManager
{
    /// <summary>
    /// A class to manage virus spawning
    /// </summary>
    public class CVirusManager
    {
        #region instance code
        private static CVirusManager instanceVMANAGER;

        public static void createInstance(int newVirusStart, int newVirusCount)
        {
            if (instanceVMANAGER == null)
            {
                instanceVMANAGER = new CVirusManager(newVirusStart, newVirusCount);
            }
        }

        public static CVirusManager getInstance
        {
            get
            {
                if (instanceVMANAGER != null)
                {
                    return instanceVMANAGER;
                }
                else
                {
                    throw new Exception("Singleton not initialised.");
                }
            }
        }
        #endregion

        #region fields and their properties
        private int firstVirus;

        public int FirstVirus
        {
            get { return firstVirus; }
            set { firstVirus = value; }
        }

        private int virusCount;

        public int VirusCount
        {
            get { return virusCount; }
            set { virusCount = value; }
        }

        // random number generator
        Random randomNum = new Random();

        public Random RandomNum
        {
            get { return randomNum; }
            set { randomNum = value; }
        }
        #endregion

        #region constructor
        public CVirusManager(int newFirstVirus, int newVirusCount)
        {
            firstVirus = newFirstVirus;
            virusCount = newVirusCount;
        }
        #endregion

        #region methods
        public void findFreeVirus(Vector2 position)
        {
            Trace.Assert(CWorld.Game1.entityList.Count != 0, "Empty list passed.");

            for (int i = firstVirus; i < firstVirus + virusCount; i++)
            {
                // checks for an inactive virus and breaks the loop
                // if one is found
                if (!CWorld.Game1.entityList[i].IsActive)
                {
                    spawnVirus((CVirus)CWorld.Game1.entityList[i], position);
                    break;
                }
            }
        }

        public void findFreeVirus(Vector2 position, Vector2 direction, GameTime time)
        {
            Trace.Assert(CWorld.Game1.entityList.Count != 0, "Empty list passed.");

            for (int i = firstVirus; i < firstVirus + virusCount; i++)
            {
                // checks for an inactive virus and breaks the loop
                // if one is found
                if (!CWorld.Game1.entityList[i].IsActive)
                {
                    spawnVirus((CVirus) CWorld.Game1.entityList[i], position, direction, time);
                    break;
                }
            }
        }

        // returns whether the ID provided is one belonging to a virus
        public bool isVirus(int ID)
        {
            return ((ID >= firstVirus) && (ID <= firstVirus + virusCount));
        }

        private void spawnVirus(CVirus virus, Vector2 position)
        {
            try
            {
                // sets the virus to spawn from the set location
                virus.CurrentPosition = position;
                virus.PreviousPosition = position;
                virus.Health = virus.OrignalHealth;
                virus.updateCollisionRectangle(virus.CurrentPosition);
                virus.IsActive = true;
                virus.Attackable = true;
                virus.CurrentTargetID = -1;
                virus.VirusBehaviour = new CRoamingVirus();
            }
            catch
            {
                throw new Exception("Failure spawning virus.");
            }
        }

        private void spawnVirus(CVirus virus, Vector2 position, Vector2 direction, GameTime time)
        {
            try
            {
                // sets the virus to spawn from the set location
                virus.CurrentPosition = position;
                virus.PreviousPosition = position;
                virus.EntityDirection = direction;
                virus.Health = virus.OrignalHealth;
                virus.updateCollisionRectangle(virus.CurrentPosition);
                virus.IsActive = true;
                virus.Attackable = true;
                virus.CurrentTargetID = -1;
                virus.VirusBehaviour = new CSpawnedVirus(direction);
                virus.StartSpawned = (float)time.TotalRealTime.TotalMilliseconds;
            }
            catch
            {
                throw new Exception("Failure spawning virus.");
            }
        }

        public int virusesRemaining()
        {
            Trace.Assert(CWorld.Game1.entityList.Count != 0, "Empty list passed.");

            // counts the number of active viruses
            int count = 0;

            for (int i = firstVirus; i < firstVirus + virusCount; i++)
            {
                // checks for an inactive virus and breaks the loop
                // if one is found
                if (CWorld.Game1.entityList[i].IsActive)
                {
                    count++;
                }
            }

            return count;
        }

        #endregion
    }
}
