﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Microsoft.Xna.Framework;
using Singleton.CCellManager;
using Singleton.CVirusManager;

namespace CWorld
{
    class CRoamingVirus : IVirus
    {
        #region fields and their properties
        private float lastDirectionChange;

        public float LastDirectionChange
        {
            get { return lastDirectionChange; }
            set { lastDirectionChange = value; }
        }

        private float DIRECTION_CHANGE_DELAY = 1000;

        public float pDIRECTION_CHANGE_DELAY
        {
            get { return DIRECTION_CHANGE_DELAY; }
            set { DIRECTION_CHANGE_DELAY = value; }
        }

        private float driftingX = 0;

        public float DriftingX
        {
            get { return driftingX; }
            set { driftingX = value; }
        }

        private float driftingY = 0;

        public float DriftingY
        {
            get { return driftingY; }
            set { driftingY = value; }
        }
        #endregion

        public void determineAction(CVirus thisVirus, GameTime time)
        {
            // holds the distance to the current target to reduce the
            // amount of times the function is called
            int distanceToTarget = thisVirus.EntityVision;

            #region finding the closest enemy to this entity
            // loops through every entity
            for (int i = 0; i < CWorld.Game1.entityList.Count; i++)
            {
                // prevents an entity checking against itself
                if (thisVirus != CWorld.Game1.entityList[i])
                {
                    // checks to see if the other entity is active
                    if (CWorld.Game1.entityList[i].IsActive)
                    {
                        // checks to see if the other entity is an enemy
                        if (thisVirus.isEnemyOf(CWorld.Game1.entityList[i].EntitySide))
                        {
                            // checks to see if the entity is range
                            if (thisVirus.inRange(CWorld.Game1.entityList[i].CurrentPosition, thisVirus.EntityVision))
                            {
                                // checks to see if the other entity can be targeted
                                if (CWorld.Game1.entityList[i].Attackable)
                                {
                                    // checks to see if this entity is closer than the current target
                                    if (thisVirus.distanceBetween(CWorld.Game1.entityList[i].CurrentPosition) < distanceToTarget)
                                    {
                                        thisVirus.CurrentTargetID = i;
                                        distanceToTarget = thisVirus.distanceBetween(CWorld.Game1.entityList[i].CurrentPosition);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            #endregion

            if (thisVirus.CurrentTargetID >= 0)
            {
                thisVirus.VirusBehaviour = new CAttackingVirus();
            }
            else
            {
                // drifting behaviour
                if (lastDirectionChange + DIRECTION_CHANGE_DELAY < (float)time.TotalRealTime.TotalMilliseconds)
                {
                    // sets X
                    if (thisVirus.CurrentPosition.X < 25)
                    {
                        driftingX = (float)(CVirusManager.getInstance.RandomNum.NextDouble());
                    }
                    else if (thisVirus.CurrentPosition.X > 725)
                    {
                        driftingX = (float)(CVirusManager.getInstance.RandomNum.NextDouble() - 1);
                    }
                    else
                    {
                        driftingX = (float)((CVirusManager.getInstance.RandomNum.NextDouble() * 2) - 1);
                    }

                    // sets Y
                    if (thisVirus.CurrentPosition.Y < 75)
                    {
                        driftingY = (float)(CVirusManager.getInstance.RandomNum.NextDouble());
                    }
                    else if (thisVirus.CurrentPosition.Y > 525)
                    {
                        driftingY = (float)(CVirusManager.getInstance.RandomNum.NextDouble() - 1);
                    }
                    else
                    {
                        driftingY = (float)((CVirusManager.getInstance.RandomNum.NextDouble() * 2) - 1);
                    }

                    DIRECTION_CHANGE_DELAY = CVirusManager.getInstance.RandomNum.Next(500, 2500);
                    lastDirectionChange = (float)time.TotalRealTime.TotalMilliseconds;
                }
                thisVirus.EntityDirection += new Vector2(driftingX, driftingY);
                thisVirus.EntityDirection += thisVirus.avoid();
            }
        }

        // collision result function
        public void collisionResult(CVirus thisVirus, int otherEntityID)
        {
            if (CCellManager.getInstance.isCell(otherEntityID))
            {
                thisVirus.IsActive = false;
            }
            else if(CWorld.Game1.entityList[otherEntityID].EntitySide != CWorldEntity.eSide.projectile)
            {
                thisVirus.takeDamage(CWorld.Game1.entityList[otherEntityID].CollisionValue);
            }
        }
    }
}
