﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.Xna.Framework;

using Singleton.CGraphics;
using Singleton.CVirusManager;
using Singleton.CProteinManager;
using Singleton.CSound;

namespace CWorld
{
    public class CVirus : CWorldEntity
    {
        #region fields and their properties
        private Vector2 entityDirection;

        public Vector2 EntityDirection
        {
            get { return entityDirection; }
            set { entityDirection = value; }
        }

        private int entityVision;

        public int EntityVision
        {
            get { return entityVision; }
            set { entityVision = value; }
        }

        private int currentTargetID;

        public int CurrentTargetID
        {
            get { return currentTargetID; }
            set { currentTargetID = value; }
        }

        private int repulsionRange;

        public int RepulsionRange
        {
            get { return repulsionRange; }
            set { repulsionRange = value; }
        }

        private IVirus virusBehaviour;

        public IVirus VirusBehaviour
        {
            get { return virusBehaviour; }
            set { virusBehaviour = value; }
        }

        private float startSpawned;

        public float StartSpawned
        {
            get { return startSpawned; }
            set { startSpawned = value; }
        }

        private float SPAWNED_DELAY;

        public float pSPAWNED_DELAY
        {
            get { return SPAWNED_DELAY; }
            set { SPAWNED_DELAY = value; }
        }

        private int experienceGiven;

        public int ExperienceGiven
        {
            get { return experienceGiven; }
            set { experienceGiven = value; }
        }
        #endregion

        public CVirus(int newGraphicID, int newHealth, int newSpeed, int newPointValue, int newCollisionValue, Vector2 newPosition,
            int newHeight, int newWidth, int newVision, int newExperienceGiven)
            : base(newGraphicID, newHealth, newSpeed, newPointValue, newCollisionValue, newPosition, newWidth, newHeight)
        {
            entitySide = eSide.virus;

            entityDirection = new Vector2(1, -1);

            // will be used for vision
            entityVision = newVision;
            repulsionRange = newVision / 2;
            currentTargetID = -1;
            startSpawned = 0;
            SPAWNED_DELAY = 1000;
            pointValue = newPointValue;
            ExperienceGiven = newExperienceGiven;

            virusBehaviour = new CRoamingVirus();
        }
        
        // the result of a collision
        public override void collisionResult(int otherEntityID)
        {
            virusBehaviour.collisionResult(this, otherEntityID);
        }

        public override bool isEnemyOf(eSide otherSide)
        {
            // checks to see if the other entity is an enemy
            return (otherSide == eSide.player || otherSide == eSide.cell || otherSide == eSide.projectile);
        }

        public override void renderEntity(float interpolateFract)
        {
            if (isActive)
            {
                Trace.Assert((interpolateFract >= 0) && (interpolateFract <= 1), "Interpolation value out of bounds."
                          , "The interpolate value is currently " + interpolateFract.ToString());

                Vector2 interVector = previousPosition;

                if (isActive)
                {
                    // only interpolates when necessary
                    if (previousPosition != currentPosition)
                    {
                        interVector += interpolateFract * (currentPosition - previousPosition);
                    }
                }

                CGraphics.get.DrawSprite(graphicID, (int)interVector.X, (int)interVector.Y);
            }
        }

        public override void takeDamage(int damage)
        {
            if (invincible == false)
            {
                entityHealth -= damage;
                if (entityHealth <= 0)
                {
                    entityHealth = 0;
                    isActive = false;
                    attackable = false;
                    CProteinManager.getInstance.spawnProtein(currentPosition, pointValue, experienceGiven, 20);
                    CSound.getInstance.PlaySoundEffect(CWorld.Game1.deathSoundId);
                }
            }
        }

        public override void update()
        {
            if (isActive)
            {
                previousPosition = currentPosition;

                // updates the collision rectangle
                updateCollisionRectangle(currentPosition);

                // normalises the direction before using it for movement
                if (entityDirection != Vector2.Zero)
                {
                    entityDirection.Normalize();
                }

                // advances the entity in the current direction times the speed
                currentPosition += (entityDirection * entitySpeed);

                // resets the direction
                entityDirection = Vector2.Zero;
            }
        }

    #region AI Functions
        // a function to determine whether another position is within this entity's
        // vision
        public bool inRange(Vector2 otherPosition, int rangeCheck)
        {
            return (distanceBetween(otherPosition) <= rangeCheck);
        }

        // a function to set the direction of the current entity
        // pass in the position to aim the entity towards
        public void determineDirection(Vector2 otherPosition)
        {
            entityDirection += otherPosition - currentPosition;
        }

        // determines the behaviour of the virus
        public override void determineAction(GameTime time)
        {
            virusBehaviour.determineAction(this, time);
        }

        // a function to avoid non attackable targets
        public Vector2 avoid()
        {
            // creates a temporary vector
            Vector2 temp = Vector2.Zero;

            for (int i = CVirusManager.getInstance.FirstVirus; i < CVirusManager.getInstance.FirstVirus + CVirusManager.getInstance.VirusCount; i++)
            {
                // ensures the entity is not comparing against itself
                if (CWorld.Game1.entityList[i] != this)
                {
                    // checks to see if the other entity is active
                    if (CWorld.Game1.entityList[i].IsActive)
                    {
                        // checks to see if the entity is range
                        if (inRange(CWorld.Game1.entityList[i].CurrentPosition, repulsionRange))
                        {
                            temp += currentPosition - CWorld.Game1.entityList[i].CurrentPosition;
                        }
                    }
                }
            }

            return temp;
        }
    #endregion
    }
}
